@@ -1,17 +1,89 @@
+#' Probabilistic sensitivity analysis.
+#'
+#' Probabilistic sensitivity analysis to correct for exposure misclassification or
+#' outcome misclassification and random error.
+#'
+#' @param exposed Exposure variable. If a variable, this variable is tabulated against.
+#' @param case Outcome variable.
+#' @param type Choice of correction for exposure or outcome misclassification.
+#' @param reps Number of replications to run.
+#' @param seca.parms List defining the sensitivity of exposure classification among those with the outcome. The first argument provides the probability distribution function (constant, uniform, triangular, trapezoidal, logit-logistic, or logit-normal) and the second its parameters as a vector:
+#' \enumerate{
+#' \item Constant: constant value,
+#' \item Uniform: min, max,
+#' \item Triangular: lower limit, upper limit, mode,
+#' \item Trapezoidal: min, lower mode, upper mode, max,
+#' \item Logit-logistic: location, scale, lower bound shift, upper bound shift,
+#' \item Logit-normal: location, scale, lower bound shift, upper bound shift.
+#' }
+#' @param seexp.parms List defining the sensitivity of exposure classification among those without the outcome.
+#' @param spca.parms List defining the specificity of exposure classification among those with the outcome.
+#' @param spexp.parms List defining the specifity of exposure classification among those without the outcome.
+#' @param corr.se Correlation between case and non-case sensitivities.
+#' @param corr.sp Correlation between case and non-case specificities.
+#' @param discard A logical scalar. In case of negative adjusted count, should the draws be discarded? If set to FALSE, negative counts are set to zero.
+#' @param alpha Significance level.
+#' @param dec Number of decimals in the printout.
+#' @param print A logical scalar. Should the results be printed?
+#'
+#' @return A list with elements:
+#' \item{obs.data}{The analysed 2 x 2 table from the observed data.}
+#' \item{obs.measures}{A table of observed relative risk and odds ratio with confidence intervals.}
+#' \item{adj.measures}{A table of corrected relative risks and odds ratios.}
+#' \item{sim.df}{Data frame of random parameters and computed values.}
+#'
+#' @references Lash, T.L., Fox, M.P, Fink, A.K., 2009 \emph{Applying Quantitative
+#' Bias Analysis to Epidemiologic Data}, pp.117--150, Springer.
+#' @examples
+#' # The data for this example come from:
+#' # Greenland S., Salvan A., Wegman D.H., Hallock M.F., Smith T.J.
+#' # A case-control study of cancer mortality at a transformer-assembly facility.
+#' # Int Arch Occup Environ Health 1994; 66(1):49-54.
+#' set.seed(123)
+#' # Exposure misclassification, non-differential
+#' probsens(matrix(c(45, 94, 257, 945),
+#' dimnames = list(c("BC+", "BC-"), c("Smoke+", "Smoke-")), nrow = 2, byrow = TRUE),
+#' type = "exposure",
+#' reps = 20000,
+#' seca.parms = list("trapezoidal", c(.75, .85, .95, 1)),
+#' spca.parms = list("trapezoidal", c(.75, .85, .95, 1)))
+#' # Exposure misclassification, differential
+#' probsens(matrix(c(45, 94, 257, 945),
+#' dimnames = list(c("BC+", "BC-"), c("Smoke+", "Smoke-")), nrow = 2, byrow = TRUE),
+#' type = "exposure",
+#' reps = 20000,
+#' seca.parms = list("trapezoidal", c(.75, .85, .95, 1)),
+#' seexp.parms = list("trapezoidal", c(.7, .8, .9, .95)),
+#' spca.parms = list("trapezoidal", c(.75, .85, .95, 1)),
+#' spexp.parms = list("trapezoidal", c(.7, .8, .9, .95)),
+#' corr.se = .8,
+#' corr.sp = .8)
+#' # Disease misclassification
+#' probsens(matrix(c(173, 602, 134, 663),
+#' dimnames = list(c("BC+", "BC-"), c("Smoke+", "Smoke-")), nrow = 2, byrow = TRUE),
+#' type = "outcome",
+#' reps = 20000,
+#' seca.parms = list("uniform", c(.8, 1)),
+#' spca.parms = list("uniform", c(.8, 1)))
+#' @export
+#' @importFrom stats median qnorm quantile runif
 probsens <- function(exposed,
                      case,
                      type = c("exposure", "outcome"),
                      reps = 1000,
-                     seca.parms = list(dist = c("uniform", "triangular",
-                                           "trapezoidal"),
+                     seca.parms = list(dist = c("constant", "uniform", "triangular",
+                                           "trapezoidal", "logit-logistic",
+                                                "logit-normal"),
                                        parms = NULL),
                      seexp.parms = NULL,
-                     spca.parms = list(dist = c("uniform", "triangular",
-                                           "trapezoidal"),
+                     spca.parms = list(dist = c("constant", "uniform", "triangular",
+                                           "trapezoidal", "logit-logistic",
+                                                "logit-normal"),
                                        parms = NULL),
                      spexp.parms = NULL,
                      corr.se = NULL,
                      corr.sp = NULL,
+                     discard = TRUE,
                      alpha = 0.05,
                      dec = 4,
                      print = TRUE){
@@ -23,6 +95,12 @@ probsens <- function(exposed,
     if(!is.list(seca.parms))
         stop('Sensitivity of exposure classification among those with the outcome should be a list.')
     else seca.parms <- seca.parms
+    if(!is.null(corr.se) && (seca.parms[[1]] == "constant" | seexp.parms[[1]] == "constant"))
+        stop('No correlated distributions with constant values.')
+    if(!is.null(corr.sp) && (spca.parms[[1]] == "constant" | spexp.parms[[1]] == "constant"))
+        stop('No correlated distributions with constant values.')
+    if(seca.parms[[1]] == "constant" & length(seca.parms[[2]]) != 1)
+        stop('For constant value, please provide a single value.')
     if(seca.parms[[1]] == "uniform" & length(seca.parms[[2]]) != 2)
         stop('For uniform distribution, please provide vector of lower and upper limits.')
     if(seca.parms[[1]] == "uniform" & seca.parms[[2]][1] >= seca.parms[[2]][2])
@@ -37,13 +115,30 @@ probsens <- function(exposed,
     if(seca.parms[[1]] == "trapezoidal" & ((seca.parms[[2]][1] > seca.parms[[2]][2]) |
                                          (seca.parms[[2]][2] > seca.parms[[2]][3]) |
                                          (seca.parms[[2]][3] > seca.parms[[2]][4])))
-        stop('Wrong arguments for your trapezoidal distribution.')    
-    if(!all(seca.parms[[2]] >= 0 & seca.parms[[2]] <= 1))
+        stop('Wrong arguments for your trapezoidal distribution.')
+    if(seca.parms[[1]] == "logit-logistic" & (length(seca.parms[[2]]) < 2 | length(seca.parms[[2]]) == 3 | length(seca.parms[[2]]) > 4))
+        stop('For logit-logistic distribution, please provide vector of location, scale, and eventually lower and upper bound limits if you want to shift and rescale the distribution.')
+    if(seca.parms[[1]] == "logit-logistic" & length(seca.parms[[2]]) == 4 &
+       ((seca.parms[[2]][3] >= seca.parms[[2]][4]) | (!all(seca.parms[[2]][3:4] >= 0 & seca.parms[[2]][3:4] <= 1))))
+        stop('For logit-logistic distribution, please provide sensible values for lower and upper bound limits (between 0 and 1; lower limit < upper limit).')
+    if(seca.parms[[1]] == "logit-logistic" & length(seca.parms[[2]]) == 2)
+        seca.parms <- list(seca.parms[[1]], c(seca.parms[[2]], c(0, 1)))
+    if(seca.parms[[1]] == "logit-normal" & (length(seca.parms[[2]]) < 2 | length(seca.parms[[2]]) == 3 | length(seca.parms[[2]]) > 4))
+        stop('For logit-normal distribution, please provide vector of location, scale, and eventually lower and upper bound limits if you want to shift and rescale the distribution.')
+    if(seca.parms[[1]] == "logit-normal" & length(seca.parms[[2]]) == 4 &
+       ((seca.parms[[2]][3] >= seca.parms[[2]][4]) | (!all(seca.parms[[2]][3:4] >= 0 & seca.parms[[2]][3:4] <= 1))))
+        stop('For logit-normal distribution, please provide sensible values for lower and upper bound limits (between 0 and 1; lower limit < upper limit).')
+    if(seca.parms[[1]] == "logit-normal" & length(seca.parms[[2]]) == 2)
+        seca.parms <- list(seca.parms[[1]], c(seca.parms[[2]], c(0, 1)))
+    if((seca.parms[[1]] == "constant" | seca.parms[[1]] == "uniform" | seca.parms[[1]] == "triangular" | seca.parms[[1]] == "trapezoidal") & !all(seca.parms[[2]] >= 0 & seca.parms[[2]] <= 1))
         stop('Sensitivity of exposure classification among those with the outcome should be between 0 and 1.')
     
     if(!is.null(seexp.parms) & !is.list(seexp.parms))
         stop('Sensitivity of exposure classification among those without the outcome should be a list.')
     else seexp.parms <- seexp.parms
+    if(!is.null(seexp.parms) && seexp.parms[[1]] == "constant" &
+       length(seexp.parms[[2]]) != 1)
+        stop('For constant value, please provide a single value.')
     if(!is.null(seexp.parms) && seexp.parms[[1]] == "uniform" &
        length(seexp.parms[[2]]) != 2)
         stop('For uniform distribution, please provide vector of lower and upper limits.')
@@ -64,13 +159,27 @@ probsens <- function(exposed,
        ((seexp.parms[[2]][1] > seexp.parms[[2]][2]) |
             (seexp.parms[[2]][2] > seexp.parms[[2]][3]) |
                 (seexp.parms[[2]][3] > seexp.parms[[2]][4])))
-        stop('Wrong arguments for your trapezoidal distribution.')    
-    if(!is.null(seexp.parms) && !all(seexp.parms[[2]] >= 0 & seexp.parms[[2]] <= 1))
+        stop('Wrong arguments for your trapezoidal distribution.')
+    if(!is.null(seexp.parms) && seexp.parms[[1]] == "logit-logistic" & (length(seexp.parms[[2]]) < 2 | length(seexp.parms[[2]]) == 3 | length(seexp.parms[[2]]) > 4))
+        stop('For logit-logistic distribution, please provide vector of location, scale, and eventually lower and upper bound limits if you want to shift and rescale the distribution.')
+    if(!is.null(seexp.parms) && seexp.parms[[1]] == "logit-logistic" & length(seexp.parms[[2]]) == 4 && ((seexp.parms[[2]][3] >= seexp.parms[[2]][4]) | (!all(seexp.parms[[2]][3:4] >= 0 & seexp.parms[[2]][3:4] <= 1))))
+        stop('For logit-logistic distribution, please provide sensible values for lower and upper bound limits (between 0 and 1; lower limit < upper limit).')
+    if(!is.null(seexp.parms) && seexp.parms[[1]] == "logit-logistic" & length(seexp.parms[[2]]) == 2)
+        seexp.parms <- list(seexp.parms[[1]], c(seexp.parms[[2]], c(0, 1)))
+    if(!is.null(seexp.parms) && seexp.parms[[1]] == "logit-normal" & (length(seexp.parms[[2]]) < 2 | length(seexp.parms[[2]]) == 3 | length(seexp.parms[[2]]) > 4))
+        stop('For logit-normal distribution, please provide vector of location, scale, and eventually lower and upper bound limits if you want to shift and rescale the distribution.')
+    if(!is.null(seexp.parms) && seexp.parms[[1]] == "logit-normal" & length(seexp.parms[[2]]) == 4 && ((seexp.parms[[2]][3] >= seexp.parms[[2]][4]) | (!all(seexp.parms[[2]][3:4] >= 0 & seexp.parms[[2]][3:4] <= 1))))
+        stop('For logit-normal distribution, please provide sensible values for lower and upper bound limits (between 0 and 1; lower limit < upper limit).')
+    if(!is.null(seexp.parms) && seexp.parms[[1]] == "logit-normal" & length(seexp.parms[[2]]) == 2)
+        seexp.parms <- list(seexp.parms[[1]], c(seexp.parms[[2]], c(0, 1)))
+    if(!is.null(seexp.parms) && (seexp.parms[[1]] == "constant" | seexp.parms[[1]] == "uniform" | seexp.parms[[1]] == "triangular" | seexp.parms[[1]] == "trapezoidal") & !all(seexp.parms[[2]] >= 0 & seexp.parms[[2]] <= 1))
         stop('Sensitivity of exposure classification among those without the outcome should be between 0 and 1.')
     
     if(!is.list(spca.parms))
         stop('Specificity of exposure classification among those with the outcome should be a list.')
     else spca.parms <- spca.parms
+    if(spca.parms[[1]] == "constant" & length(spca.parms[[2]]) != 1)
+        stop('For constant value, please provide a single value.')
     if(spca.parms[[1]] == "uniform" & length(spca.parms[[2]]) != 2)
         stop('For uniform distribution, please provide vector of lower and upper limits.')
     if(spca.parms[[1]] == "uniform" & spca.parms[[2]][1] >= spca.parms[[2]][2])
@@ -86,12 +195,29 @@ probsens <- function(exposed,
                                          (spca.parms[[2]][2] > spca.parms[[2]][3]) |
                                          (spca.parms[[2]][3] > spca.parms[[2]][4])))
         stop('Wrong arguments for your trapezoidal distribution.')    
-    if(!all(spca.parms[[2]] >= 0 & spca.parms[[2]] <= 1))
+    if(spca.parms[[1]] == "logit-logistic" & (length(spca.parms[[2]]) < 2 | length(spca.parms[[2]]) == 3 | length(spca.parms[[2]]) > 4))
+        stop('For logit-logistic distribution, please provide vector of location, scale, and eventually lower and upper bound limits if you want to shift and rescale the distribution.')
+    if(spca.parms[[1]] == "logit-logistic" & length(spca.parms[[2]]) == 4 &
+       ((spca.parms[[2]][3] >= spca.parms[[2]][4]) | (!all(spca.parms[[2]][3:4] >= 0 & spca.parms[[2]][3:4] <= 1))))
+        stop('For logit-logistic distribution, please provide sensible values for lower and upper bound limits (between 0 and 1; lower limit < upper limit).')
+    if(spca.parms[[1]] == "logit-logistic" & length(spca.parms[[2]]) == 2)
+        spca.parms <- list(spca.parms[[1]], c(spca.parms[[2]], c(0, 1)))
+    if(spca.parms[[1]] == "logit-normal" & (length(spca.parms[[2]]) < 2 | length(spca.parms[[2]]) == 3 | length(spca.parms[[2]]) > 4))
+        stop('For logit-normal distribution, please provide vector of location, scale, and eventually lower and upper bound limits if you want to shift and rescale the distribution.')
+    if(spca.parms[[1]] == "logit-normal" & length(spca.parms[[2]]) == 4 &
+       ((spca.parms[[2]][3] >= spca.parms[[2]][4]) | (!all(spca.parms[[2]][3:4] >= 0 & spca.parms[[2]][3:4] <= 1))))
+        stop('For logit-normal distribution, please provide sensible values for lower and upper bound limits (between 0 and 1; lower limit < upper limit).')
+    if(spca.parms[[1]] == "logit-normal" & length(spca.parms[[2]]) == 2)
+        spca.parms <- list(spca.parms[[1]], c(spca.parms[[2]], c(0, 1)))
+    if((spca.parms[[1]] == "constant" | spca.parms[[1]] == "uniform" | spca.parms[[1]] == "triangular" | spca.parms[[1]] == "trapezoidal") & !all(spca.parms[[2]] >= 0 & spca.parms[[2]] <= 1))
         stop('Specificity of exposure classification among those with the outcome should be between 0 and 1.')
     
     if(!is.null(spexp.parms) & !is.list(spexp.parms))
         stop('Specificity of exposure classification among those without the outcome should be a list.')
     else spexp.parms <- spexp.parms
+    if(!is.null(spexp.parms) && spexp.parms[[1]] == "constant" &
+       length(spexp.parms[[2]]) != 1)
+        stop('For constant value, please provide a single value.')
     if(!is.null(spexp.parms) && spexp.parms[[1]] == "uniform" &
        length(spexp.parms[[2]]) != 2)
         stop('For uniform distribution, please provide vector of lower and upper limits.')
@@ -113,7 +239,19 @@ probsens <- function(exposed,
             (spexp.parms[[2]][2] > spexp.parms[[2]][3]) |
                 (spexp.parms[[2]][3] > spexp.parms[[2]][4])))
         stop('Wrong arguments for your trapezoidal distribution.')    
-    if(!is.null(spexp.parms) && !all(spexp.parms[[2]] >= 0 & spexp.parms[[2]] <= 1))
+    if(!is.null(spexp.parms) && spexp.parms[[1]] == "logit-logistic" & (length(spexp.parms[[2]]) < 2 | length(spexp.parms[[2]]) == 3 | length(spexp.parms[[2]]) > 4))
+        stop('For logit-logistic distribution, please provide vector of location, scale, and eventually lower and upper bound limits if you want to shift and rescale the distribution.')
+    if(!is.null(spexp.parms) && spexp.parms[[1]] == "logit-logistic" & length(spexp.parms[[2]]) == 4 && ((spexp.parms[[2]][3] >= spexp.parms[[2]][4]) | (!all(spexp.parms[[2]][3:4] >= 0 & spexp.parms[[2]][3:4] <= 1))))
+        stop('For logit-logistic distribution, please provide sensible values for lower and upper bound limits (between 0 and 1; lower limit < upper limit).')
+    if(!is.null(seexp.parms) && spexp.parms[[1]] == "logit-logistic" & length(spexp.parms[[2]]) == 2)
+        spexp.parms <- list(spexp.parms[[1]], c(spexp.parms[[2]], c(0, 1)))
+    if(!is.null(spexp.parms) && spexp.parms[[1]] == "logit-normal" & (length(spexp.parms[[2]]) < 2 | length(spexp.parms[[2]]) == 3 | length(spexp.parms[[2]]) > 4))
+        stop('For logit-normal distribution, please provide vector of location, scale, and eventually lower and upper bound limits if you want to shift and rescale the distribution.')
+    if(!is.null(spexp.parms) && spexp.parms[[1]] == "logit-normal" & length(spexp.parms[[2]]) == 4 && ((spexp.parms[[2]][3] >= spexp.parms[[2]][4]) | (!all(spexp.parms[[2]][3:4] >= 0 & spexp.parms[[2]][3:4] <= 1))))
+        stop('For logit-normal distribution, please provide sensible values for lower and upper bound limits (between 0 and 1; lower limit < upper limit).')
+    if(!is.null(spexp.parms) && spexp.parms[[1]] == "logit-normal" & length(spexp.parms[[2]]) == 2)
+        spexp.parms <- list(spexp.parms[[1]], c(spexp.parms[[2]], c(0, 1)))
+    if(!is.null(spexp.parms) && (spexp.parms[[1]] == "constant" | spexp.parms[[1]] == "uniform" | spexp.parms[[1]] == "triangular" | spexp.parms[[1]] == "trapezoidal") & !all(spexp.parms[[2]] >= 0 & spexp.parms[[2]] <= 1))
         stop('Specificity of exposure classification among those without the outcome should be between 0 and 1.')
     
     if(!is.null(seexp.parms) & (is.null(spca.parms) | is.null(spexp.parms) |
@@ -156,8 +294,24 @@ probsens <- function(exposed,
     lci.obs.or <- exp(log(obs.or) - qnorm(1 - alpha/2) * se.log.obs.or)
     uci.obs.or <- exp(log(obs.or) + qnorm(1 - alpha/2) * se.log.obs.or)
 
+    logitlog.dstr <- function(sesp) {
+        u <- runif(sesp[[1]])
+        w <- sesp[[2]] + sesp[[3]] * (log(u / (1 - u)))
+        p <- sesp[[4]] + (sesp[[5]] - sesp[[4]]) * exp(w) / (1 + exp(w))
+        return(p)
+    }
+    logitnorm.dstr <- function(sesp) {
+        u <- runif(sesp[[1]])
+        w <- sesp[[2]] + sesp[[3]] * qnorm(u)
+        p <- sesp[[4]] + (sesp[[5]] - sesp[[4]]) * exp(w) / (1 + exp(w))
+        return(p)
+    }
+    
     if (is.null(seexp.parms) & !is.null(spca.parms) & is.null(spexp.parms) &
         is.null(corr.se) & is.null(corr.sp)) {
+        if (seca.parms[[1]] == "constant") {
+            draws[, 1] <- seca.parms[[2]]
+        }
         if (seca.parms[[1]] == "uniform") {
             draws[, 1] <- do.call(runif, as.list(seca))
             }
@@ -167,7 +321,16 @@ probsens <- function(exposed,
         if (seca.parms[[1]] == "trapezoidal") {
             draws[, 1] <- do.call(trapezoid::rtrapezoid, as.list(seca))
             }
+        if (seca.parms[[1]] == "logit-logistic") {
+            draws[, 1] <- logitlog.dstr(seca)
+            }
+        if (seca.parms[[1]] == "logit-normal") {
+            draws[, 1] <- logitnorm.dstr(seca)
+            }
         draws[, 2] <- draws[, 1]
+        if (spca.parms[[1]] == "constant") {
+            draws[, 2] <- spca.parms[[2]]
+        }
         if (spca.parms[[1]] == "uniform") {
             draws[, 3] <- do.call(runif, as.list(spca))
             }
@@ -177,6 +340,12 @@ probsens <- function(exposed,
         if (spca.parms[[1]] == "trapezoidal") {
             draws[, 3] <- do.call(trapezoid::rtrapezoid, as.list(spca))
             }
+        if (spca.parms[[1]] == "logit-logistic") {
+            draws[, 3] <- logitlog.dstr(spca)
+            }
+        if (spca.parms[[1]] == "logit-normal") {
+            draws[, 3] <- logitnorm.dstr(spca)
+            }
         draws[, 4] <- draws[, 3]
     } else {
         corr.draws[, 1:6] <- apply(corr.draws[, 1:6],
@@ -218,6 +387,14 @@ probsens <- function(exposed,
                              seca.parms[[2]][4] - sqrt(abs(2 * (seca.parms[[2]][4] - seca.parms[[2]][3]) * (draws[, 1] - seca.parms[[2]][3]))),
                              draws[, 1])
     }
+    if (seca.parms[[1]] == "logit-logistic") {
+        seca.w <- seca.parms[[2]][1] + (seca.parms[[2]][2] * log(corr.draws[, 7] / (1 - corr.draws[, 7])))
+        draws[, 1] <- seca.parms[[2]][3] + (seca.parms[[2]][4] - seca.parms[[2]][3]) * exp(seca.w) / (1 + exp(seca.w))
+    }
+    if (seca.parms[[1]] == "logit-normal") {
+        seca.w <- seca.parms[[2]][1] + (seca.parms[[2]][2] * qnorm(corr.draws[, 7]))
+        draws[, 1] <- seca.parms[[2]][3] + (seca.parms[[2]][4] - seca.parms[[2]][3]) * exp(seca.w) / (1 + exp(seca.w))
+    }
     if (seexp.parms[[1]] == "uniform") {
         draws[, 2] <- seexp.parms[[2]][2] -
             (seexp.parms[[2]][2] - seexp.parms[[2]][1]) * corr.draws[, 8]
@@ -242,6 +419,14 @@ probsens <- function(exposed,
                              seexp.parms[[2]][4] - sqrt(abs(2 * (seexp.parms[[2]][4] - seexp.parms[[2]][3]) * (draws[, 2] - seexp.parms[[2]][3]))),
                              draws[, 2])
     }
+    if (seexp.parms[[1]] == "logit-logistic") {
+        seexp.w <- seexp.parms[[2]][1] + (seexp.parms[[2]][2] * log(corr.draws[, 8] / (1 - corr.draws[, 8])))
+        draws[, 2] <- seexp.parms[[2]][3] + (seexp.parms[[2]][4] - seexp.parms[[2]][3]) * exp(seexp.w) / (1 + exp(seexp.w))
+    }
+    if (seexp.parms[[1]] == "logit-normal") {
+        seexp.w <- seexp.parms[[2]][1] + (seexp.parms[[2]][2] * qnorm(corr.draws[, 8]))
+        draws[, 2] <- seexp.parms[[2]][3] + (seexp.parms[[2]][4] - seexp.parms[[2]][3]) * exp(seexp.w) / (1 + exp(seexp.w))
+    }
     if (spca.parms[[1]] == "uniform") {
         draws[, 3] <- spca.parms[[2]][2] -
             (spca.parms[[2]][2] - spca.parms[[2]][1]) * corr.draws[, 9]
@@ -266,6 +451,14 @@ probsens <- function(exposed,
                              spca.parms[[2]][4] - sqrt(abs(2 * (spca.parms[[2]][4] - spca.parms[[2]][3]) * (draws[, 3] - spca.parms[[2]][3]))),
                              draws[, 3])
     }
+    if (spca.parms[[1]] == "logit-logistic") {
+        spca.w <- spca.parms[[2]][1] + (spca.parms[[2]][2] * log(corr.draws[, 9] / (1 - corr.draws[, 9])))
+        draws[, 3] <- spca.parms[[2]][3] + (spca.parms[[2]][4] - spca.parms[[2]][3]) * exp(spca.w) / (1 + exp(spca.w))
+    }
+    if (spca.parms[[1]] == "logit-normal") {
+        spca.w <- spca.parms[[2]][1] + (spca.parms[[2]][2] * qnorm(corr.draws[, 9]))
+        draws[, 3] <- spca.parms[[2]][3] + (spca.parms[[2]][4] - spca.parms[[2]][3]) * exp(spca.w) / (1 + exp(spca.w))
+    }
     if (spexp.parms[[1]] == "uniform") {
         draws[, 4] <- spexp.parms[[2]][2] -
             (spexp.parms[[2]][2] - spexp.parms[[2]][1]) * corr.draws[, 10]
@@ -290,6 +483,14 @@ probsens <- function(exposed,
                              spexp.parms[[2]][4] - sqrt(abs(2 * (spexp.parms[[2]][4] - spexp.parms[[2]][3]) * (draws[, 4] - spexp.parms[[2]][3]))),
                              draws[, 4])
     }
+    if (spexp.parms[[1]] == "logit-logistic") {
+        spexp.w <- spexp.parms[[2]][1] + (spexp.parms[[2]][2] * log(corr.draws[, 10] / (1 - corr.draws[, 10])))
+        draws[, 4] <- spexp.parms[[2]][3] + (spexp.parms[[2]][4] - spexp.parms[[2]][3]) * exp(spexp.w) / (1 + exp(spexp.w))
+    }
+    if (spexp.parms[[1]] == "logit-normal") {
+        spexp.w <- spexp.parms[[2]][1] + (spexp.parms[[2]][2] * qnorm(corr.draws[, 10]))
+        draws[, 4] <- spexp.parms[[2]][3] + (spexp.parms[[2]][4] - spexp.parms[[2]][3]) * exp(spexp.w) / (1 + exp(spexp.w))
+    }
     }
 
     draws[, 11] <- runif(reps)
@@ -308,21 +509,38 @@ probsens <- function(exposed,
         draws[, 10] <- (draws[, 5]/draws[, 7]) / (draws[, 6]/draws[, 8])
 
         draws[, 9] <- ifelse(draws[, 5] < 1 |
-                               draws[, 6] < 1 |
-                                 draws[, 7] < 1 |
-                                   draws[, 8] < 1, NA, draws[, 9])
+                                 draws[, 6] < 1 |
+                                     draws[, 7] < 1 |
+                                         draws[, 8] < 1, NA, draws[, 9])
         draws[, 10] <- ifelse(draws[, 5] < 1 |
-                               draws[, 6] < 1 |
-                                 draws[, 7] < 1 |
-                                   draws[, 8] < 1, NA, draws[, 10])
+                                 draws[, 6] < 1 |
+                                     draws[, 7] < 1 |
+                                         draws[, 8] < 1, NA, draws[, 10])
         draws[, 9] <- ifelse(draws[, 2] < (c / (c + d)) |
-                             draws[, 4] < (d / (c + d)) |
-                             draws[, 1] < (a / (a + b)) |
-                             draws[, 3] < (b / (a + b)), NA, draws[, 9])
+                                 draws[, 4] < (d / (c + d)) |
+                                     draws[, 1] < (a / (a + b)) |
+                                         draws[, 3] < (b / (a + b)), NA, draws[, 9])
         draws[, 10] <- ifelse(draws[, 2] < (c / (c + d)) |
-                              draws[, 4] < (d / (c + d)) |
-                              draws[, 1] < (a / (a + b)) |
-                              draws[, 3] < (b / (a + b)), NA, draws[, 10])
+                                  draws[, 4] < (d / (c + d)) |
+                                      draws[, 1] < (a / (a + b)) |
+                                          draws[, 3] < (b / (a + b)), NA, draws[, 10])
+        if(all(is.na(draws[, 9])) | all(is.na(draws[, 10])))
+            warning('Prior Se/Sp distributions lead to all negative adjusted counts.')
+        if (discard) {
+            if(sum(is.na(draws[, 9])) > 0)
+                message('Chosen prior Se/Sp distributions lead to ',
+                        sum(is.na(draws[, 9])),
+                        ' negative adjusted counts which were discarded.')
+        }
+        else {
+            if(sum(is.na(draws[, 9])) > 0) {
+                message('Chosen prior Se/Sp distributions lead to ',
+                        sum(is.na(draws[, 9])),
+                        ' negative adjusted counts which were set to zero.')
+            }
+            draws[, 9] <- ifelse(is.na(draws[, 9]), 0, draws[, 9])
+            draws[, 10] <- ifelse(is.na(draws[, 10]), 0, draws[, 10])
+        }
 
         draws[, 12] <- exp(log(draws[, 9]) -
                                qnorm(draws[, 11]) *
@@ -350,44 +568,42 @@ probsens <- function(exposed,
             rownames(tab) <- paste("Row", 1:2)
         if (is.null(colnames(tab)))
             colnames(tab) <- paste("Col", 1:2)
-        if (print)
+        if (print) {
             cat("Observed Data:",
                 "\n--------------", 
                 "\nOutcome   :", rownames(tab)[1],
                 "\nComparing :", colnames(tab)[1], "vs.", colnames(tab)[2], "\n\n")
-        if (print) 
             print(round(tab, dec))
-        if (print) 
             cat("\n")
+            }
         rmat <- rbind(c(obs.rr, lci.obs.rr, uci.obs.rr),
                       c(obs.or, lci.obs.or, uci.obs.or))
         rownames(rmat) <- c(" Observed Relative Risk:", "    Observed Odds Ratio:")
         colnames(rmat) <- c("     ", paste(100 * (1 - alpha), "% conf.", 
                                                sep = ""), "interval")
-        if (print)
+        if (print) {
             cat("Observed Measures of Exposure-Outcome Relationship:",
                 "\n-----------------------------------------------------------------------------------\n\n")
-        if (print) 
             print(round(rmat, dec))
-        if (print)
             cat("\n")
+            }
         rmatc <- rbind(corr.rr, corr.or, tot.rr, tot.or)
         rownames(rmatc) <- c("           Relative Risk -- systematic error:",
                              "              Odds Ratio -- systematic error:",
                              "Relative Risk -- systematic and random error:",
                              "   Odds Ratio -- systematic and random error:")
         colnames(rmatc) <- c("Median", "2.5th percentile", "97.5th percentile")
-        if (print)
+        if (print) {
             print(round(rmatc, dec))
-        if (print)
             cat("\nBias Parameters:",
                 "\n----------------\n\n")
-        if (print)
             cat("   Se|Cases:", seca.parms[[1]], "(", seca.parms[[2]], ")",
                 "\n   Sp|Cases:", spca.parms[[1]], "(", spca.parms[[2]], ")",
                 "\nSe|No-cases:", seexp.parms[[1]], "(", seexp.parms[[2]], ")",
                 "\nSp|No-cases:", spexp.parms[[1]], "(", spexp.parms[[2]], ")",
+                "\nDiscard negative adjusted counts:", discard,
                 "\n")
+            }
         }
 
     if (type == "outcome") {
@@ -418,6 +634,23 @@ probsens <- function(exposed,
                               draws[, 4] < (d / (c + d)) |
                               draws[, 1] < (a / (a + b)) |
                               draws[, 3] < (b / (a + b)), NA, draws[, 10])
+        if(all(is.na(draws[, 9])) | all(is.na(draws[, 10])))
+            warning('Prior Se/Sp distributions lead to all negative adjusted counts.')
+        if (discard) {
+            if(sum(is.na(draws[, 9])) > 0)
+                message('Chosen prior Se/Sp distributions lead to ',
+                        sum(is.na(draws[, 9])),
+                        ' negative adjusted counts which were discarded.')
+        }
+        else {
+            if(sum(is.na(draws[, 9])) > 0) {
+                message('Chosen prior Se/Sp distributions lead to ',
+                        sum(is.na(draws[, 9])),
+                        ' negative adjusted counts which were set to zero.')
+            }
+            draws[, 9] <- ifelse(is.na(draws[, 9]), 0, draws[, 9])
+            draws[, 10] <- ifelse(is.na(draws[, 10]), 0, draws[, 10])
+        }
 
         draws[, 12] <- exp(log(draws[, 9]) -
                                qnorm(draws[, 11]) *
@@ -445,44 +678,42 @@ probsens <- function(exposed,
             rownames(tab) <- paste("Row", 1:2)
         if (is.null(colnames(tab)))
             colnames(tab) <- paste("Col", 1:2)
-        if (print)
+        if (print) {
             cat("Observed Data:",
                 "\n--------------", 
                 "\nOutcome   :", rownames(tab)[1],
                 "\nComparing :", colnames(tab)[1], "vs.", colnames(tab)[2], "\n\n")
-        if (print) 
             print(round(tab, dec))
-        if (print) 
             cat("\n")
+            }
         rmat <- rbind(c(obs.rr, lci.obs.rr, uci.obs.rr),
                       c(obs.or, lci.obs.or, uci.obs.or))
         rownames(rmat) <- c(" Observed Relative Risk:", "    Observed Odds Ratio:")
         colnames(rmat) <- c("     ", paste(100 * (1 - alpha), "% conf.", 
                                                sep = ""), "interval")
-        if (print)
+        if (print) {
             cat("Observed Measures of Exposure-Outcome Relationship:",
                 "\n-----------------------------------------------------------------------------------\n\n")
-        if (print) 
             print(round(rmat, dec))
-        if (print)
             cat("\n")
+            }
         rmatc <- rbind(corr.rr, corr.or, tot.rr, tot.or)
         rownames(rmatc) <- c("           Relative Risk -- systematic error:",
                              "              Odds Ratio -- systematic error:",
                              "Relative Risk -- systematic and random error:",
                              "   Odds Ratio -- systematic and random error:")
         colnames(rmatc) <- c("Median", "2.5th percentile", "97.5th percentile")
-        if (print)
+        if (print) {
             print(round(rmatc, dec))
-        if (print)
             cat("\nBias Parameters:",
                 "\n----------------\n\n")
-        if (print)
             cat("    Se|Exposed:", seca.parms[[1]], "(", seca.parms[[2]], ")",
                 "\n    Sp|Exposed:", spca.parms[[1]], "(", spca.parms[[2]], ")",
                 "\nSe|Non-exposed:", seexp.parms[[1]], "(", seexp.parms[[2]], ")",
                 "\nSp|Non-exposed:", spexp.parms[[1]], "(", spexp.parms[[2]], ")",
+                "\nDiscard negative adjusted counts:", discard,
                 "\n")
+            }
         }
     invisible(list(obs.data = tab,
                    obs.measures = rmat, 
